# Generated by Django 5.2.4 on 2025-09-10 03:24

import django_revision.revision_field
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pharmacy", "0087_alter_storagebinitem_stock"),
    ]

    operations = [
        migrations.AlterField(
            model_name="allocation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="assignment",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="confirmation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsite",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="confirmationatsiteitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="container",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="dispense",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="dispenseitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="dosageguideline",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="edcpermissions",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="formulation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalallocation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalassignment",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsite",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconfirmationatsiteitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalcontainer",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispense",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldispenseitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldosageguideline",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalformulation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicallocation",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmedication",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalorderitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalproduct",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceive",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalreceiveitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrepackrequest",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrx",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalrxrefill",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstock",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockadjustment",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequest",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstockrequestitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransfer",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstocktransferitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebin",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalstoragebinitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsupplier",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="lot",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="medication",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="orderitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="product",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="receive",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="receiveitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="repackrequest",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="room",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="rx",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="rxrefill",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="scanduplicates",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="shelf",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="stock",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="stockadjustment",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequest",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="stockrequestitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="stocktransfer",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="stocktransferitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="storagebin",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="storagebinitem",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="supplier",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                null=True,
                verbose_name="Revision",
            ),
        ),
    ]



from django.db import models

from edc_model.models import BaseUuidModel


class Manager(models.Manager):
    use_in_migrations = True


class ScanDuplicates(BaseUuidModel):

    identifier = models.CharField(
        max_length=36,
        unique=True,
    )

    objects = Manager()

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Scan Duplicates"
        verbose_name_plural = "Scan Duplicates"



# Generated by Django 2.2.2 on 2019-06-27 20:57

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_action_item", "0018_auto_20190626_1629")]

    operations = [
        migrations.AddIndex(
            model_name="actionitem",
            index=models.Index(
                fields=["action_identifier", "status", "report_datetime"],
                name="edc_action__action__b19a08_idx",
            ),
        )
    ]



SEVEN_DAY_CLINIC = "7-day-clinic"
SIX_DAY_CLINIC = "6-day-clinic"
FIVE_DAY_CLINIC = "5-day-clinic"
TU_WE_TH_CLINIC = "3-day-clinic"
FOUR_DAY_CLINIC = "4-day-clinic"
HEALTH_FACILITY = "health_facility"



from django.db import models
from django.utils import timezone
from sequences import get_next_value

from edc_model.models import BaseUuidModel, HistoricalRecords

from ...exceptions import StockTransferError
from .stock import Stock
from .stock_transfer import StockTransfer


class Manager(models.Manager):
    use_in_migrations = True


class StockTransferItem(BaseUuidModel):
    """A model to track allocated stock transfers from location A
    to location B.
    """

    transfer_item_identifier = models.CharField(
        max_length=36,
        unique=True,
        null=True,
        blank=True,
        help_text="A sequential unique identifier set by the EDC",
    )

    transfer_item_datetime = models.DateTimeField(default=timezone.now)

    stock_transfer = models.ForeignKey(StockTransfer, on_delete=models.PROTECT)

    stock = models.OneToOneField(
        Stock,
        on_delete=models.PROTECT,
        null=True,
        blank=False,
        limit_choices_to={"allocation__isnull": False},
    )

    objects = Manager()

    history = HistoricalRecords()

    def __str__(self):
        return self.transfer_item_identifier

    def save(self, *args, **kwargs):
        if not self.transfer_item_identifier:
            self.transfer_item_identifier = f"{get_next_value(self._meta.label_lower):06d}"
            if self.stock.location != self.stock_transfer.from_location:
                raise StockTransferError(
                    "Location mismatch. Current stock location must match "
                    "`from_location. Perhaps catch this in the form"
                )
        super().save(*args, **kwargs)

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Stock transfer item"
        verbose_name_plural = "Stock transfer items"



from django.db import models

from edc_lab.constants import EQ
from edc_reportable import PERCENT

from ..constants import GLUCOSE_HIGH_READING


class Hba1cModelMixin(models.Model):
    """A model mixin of fields for the HbA1c"""

    hba1c_value = models.DecimalField(
        verbose_name="HbA1c value",
        max_digits=8,
        decimal_places=2,
        null=True,
        blank=True,
        help_text=f"In percent. A `HIGH` reading may be entered as {GLUCOSE_HIGH_READING}",
    )

    hba1c_quantifier = models.CharField(
        verbose_name="HbA1c quantifier",
        max_length=10,
        default=EQ,
        editable=False,
    )

    hba1c_units = models.CharField(
        verbose_name="HbA1c units",
        max_length=15,
        default=PERCENT,
        blank=True,
    )

    hba1c_datetime = models.DateTimeField(
        verbose_name="Date/time HbA1c measured",
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-08-07 12:43
from __future__ import unicode_literals

import django_audit_fields.fields.uuid_auto_field
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("edc_identifier", "0003_auto_20160625_0938")]

    operations = [
        migrations.AlterField(
            model_name="identifiertracker",
            name="id",
            field=django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                blank=True,
                editable=False,
                help_text="System auto field. UUID primary key.",
                primary_key=True,
                serialize=False,
            ),
        ),
        migrations.AlterField(
            model_name="subjectidentifier",
            name="id",
            field=django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                blank=True,
                editable=False,
                help_text="System auto field. UUID primary key.",
                primary_key=True,
                serialize=False,
            ),
        ),
    ]



from edc_navbar import Navbar, NavbarItem, site_navbars

navbar = Navbar(name="edc_export")

navbar.register(
    NavbarItem(
        name="export",
        label="Export",
        fa_icon="fa-file-export",
        url_name="edc_export:home_url",
        codename="edc_navbar.nav_export",
    )
)

navbar.register(
    NavbarItem(
        name="data_request",
        label="Export Admin",
        url_name="edc_export:admin:index",
        codename="edc_navbar.nav_export",
    )
)


site_navbars.register(navbar)



import pandas as pd

from edc_constants.constants import FEMALE, MALE

from ..constants import (
    MEDIAN_IQR,
    MEDIAN_RANGE,
    N_ONLY,
    N_WITH_COL_PROP,
    N_WITH_ROW_PROP,
)
from ..row import RowDefinition, RowDefinitions
from ..table import Table


class BmiTable(Table):

    colname = "calculated_bmi_value"

    def __init__(self, main_df: pd.DataFrame = None):
        super().__init__(
            colname="calculated_bmi_value",
            main_df=main_df,
            title="BMI categories (kg/m2)",
        )

    @property
    def row_definitions(self) -> RowDefinitions:
        df_tmp = self.main_df.copy()
        row_defs = RowDefinitions(reverse_rows=False)
        row0 = RowDefinition(
            title=self.title,
            label=self.default_sublabel,
            condition=(df_tmp["gender"].notna()),
            columns={FEMALE: (N_ONLY, 2), MALE: (N_ONLY, 2), "All": (N_ONLY, 2)},
            drop=False,
        )
        row_defs.add(row0)
        columns = {
            FEMALE: (N_WITH_COL_PROP, 2),
            MALE: (N_WITH_COL_PROP, 2),
            "All": (N_WITH_ROW_PROP, 2),
        }
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="Less than 18.5",
                condition=(df_tmp[self.colname] < 18.5),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="18.5-24.9",
                condition=(df_tmp[self.colname] >= 18.5) & (df_tmp[self.colname] < 25.0),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="25.0-29.9",
                condition=(df_tmp[self.colname] >= 25.0) & (df_tmp[self.colname] < 30.0),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="30.0-39.9",
                condition=(df_tmp[self.colname] >= 30.0) & (df_tmp[self.colname] < 40.0),
                columns=columns,
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="40 or above",
                condition=(df_tmp[self.colname] >= 40.0),
                columns=columns,
                drop=False,
            )
        )
        cond = df_tmp[self.colname].isna()
        if len(df_tmp[cond]) > 0:
            row_defs.add(
                RowDefinition(
                    colname="calculated_bmi_value",
                    label="not measured",
                    condition=cond,
                    columns=columns,
                    drop=False,
                )
            )
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="Median (IQR)",
                condition=(df_tmp["gender"].notna()),
                columns={
                    FEMALE: (MEDIAN_IQR, 2),
                    MALE: (MEDIAN_IQR, 2),
                    "All": (MEDIAN_IQR, 2),
                },
                drop=False,
            )
        )
        row_defs.add(
            RowDefinition(
                colname="calculated_bmi_value",
                label="Median (range)",
                condition=(df_tmp["gender"].notna()),
                columns={
                    FEMALE: (MEDIAN_RANGE, 2),
                    MALE: (MEDIAN_RANGE, 2),
                    "All": (MEDIAN_RANGE, 2),
                },
                drop=False,
            )
        )
        return row_defs



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_locator",
            "0040_alter_historicalsubjectlocator_action_identifier_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



from datetime import date, datetime

from .egfr import Egfr


def get_egfr_for_subject(
    creatinine_value: float = None,
    creatinine_units: str = None,
    assay_datetime: datetime = None,
    dob: date = None,
    gender: str = None,
    ethnicity: str = None,
    report_datetime: datetime = None,
    baseline_egfr_value: float | None = None,
    value_threshold: float | None = None,
    percent_drop_threshold: float | None = None,
):
    return Egfr(
        percent_drop_threshold=percent_drop_threshold or 20.0000,
        dob=dob,
        gender=gender,
        ethnicity=ethnicity,
        value_threshold=value_threshold or 45.0000,
        report_datetime=report_datetime,
        baseline_egfr_value=baseline_egfr_value,
        formula_name="ckd-epi",
        reference_range_collection_name="meta",
        creatinine_units=creatinine_units,
        creatinine_value=creatinine_value,
        assay_datetime=assay_datetime,
    )



from django.conf import settings

from .dashboard_templates import dashboard_templates
from .url_names import url_names


class DashboardMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        try:
            request.url_name_data
        except AttributeError:
            request.url_name_data = url_names.registry
        try:
            request.template_data
        except AttributeError:
            request.template_data = {}
        response = self.get_response(request)
        return response

    def process_view(self, request, *args):
        """Adds/Updates references to urls and templates."""
        template_data = dashboard_templates
        try:
            template_data.update(settings.DASHBOARD_BASE_TEMPLATES)
        except AttributeError:
            pass
        request.template_data.update(**template_data)

    def process_template_response(self, request, response):
        return response



from __future__ import annotations

from django.urls import NoReverseMatch, reverse

from ..utils import get_value_from_lookup_string


class ModelAdminRedirectAllToChangelistMixinError(Exception):
    pass


class ModelAdminRedirectAllToChangelistMixin:
    """Redirects save, delete, cancel to the changelist.

    Overrides add/change views to intercept the post_save url and
    manipulate the redirect on cancel/delete.

    Important: Declare with ModelAdminNextUrlRedirectMixin and
        ModelAdminRedirectOnDeleteMixin.
    """

    changelist_url = None
    search_querystring_attr = "q"  # e.g ?q=12345
    change_search_field_name = None  # e.g. `screening_identifier` from model
    add_search_field_name = None

    def get_changelist_url(self, request):
        return self.changelist_url

    def redirect_url(self, request, obj, post_url_continue=None) -> str | None:
        if request.GET.dict().get(self.next_querystring_attr):
            return super().redirect_url(request, obj, post_url_continue=post_url_continue)
        return self.response_post_save_change(request, obj)

    def response_post_save_change(self, request, obj):
        try:
            url = reverse(self.get_changelist_url(request))
        except NoReverseMatch as e:
            raise ModelAdminRedirectAllToChangelistMixinError(e)
        value = get_value_from_lookup_string(self.change_search_field_name, obj=obj)
        if obj and value:
            url = f"{url}?q={value}"
        return url

    def get_post_full_url_on_delete(self, request):
        return self.get_changelist_url(request)

    def post_url_on_delete_querystring_kwargs(self, request, obj) -> dict:
        q = get_value_from_lookup_string(self.change_search_field_name, obj=obj)
        return dict(q=q)

    def add_view(self, request, form_url="", extra_context=None):
        q = get_value_from_lookup_string(
            self.add_search_field_name, request=request
        ) or get_value_from_lookup_string(self.change_search_field_name, request=request)
        extra_context = extra_context or {}
        extra_context.update(
            cancel_url=self.get_changelist_url(request),
            cancel_url_querystring_data={"q": q},
        )
        return super().add_view(request, form_url=form_url, extra_context=extra_context)

    def change_view(self, request, object_id, form_url="", extra_context=None):
        q = get_value_from_lookup_string(
            self.change_search_field_name,
            obj=self.model.objects.get(id=object_id),
        )
        extra_context = extra_context or {}
        extra_context.update(
            cancel_url=self.get_changelist_url(request),
            cancel_url_querystring_data={"q": q},
        )
        return super().change_view(
            request, object_id, form_url=form_url, extra_context=extra_context
        )



# Generated by Django 2.2.6 on 2019-10-22 01:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_auth", "0008_auto_20191022_0134")]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="alternate_email",
            field=models.EmailField(
                blank=True, max_length=254, verbose_name="Alternate email address"
            ),
        )
    ]



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps

if TYPE_CHECKING:
    from django.contrib.sites.models import Site


def get_site_model_cls() -> Site:
    return django_apps.get_model("sites.site")



from django.db import models


class AddressModelMixin(models.Model):
    address_one = models.CharField(max_length=255, default="", blank=True)

    address_two = models.CharField(max_length=255, default="", blank=True)

    city = models.CharField(max_length=255, default="", blank=True)

    postal_code = models.CharField(max_length=255, default="", blank=True)

    state = models.CharField(max_length=255, default="", blank=True)

    country = models.CharField(max_length=255, default="", blank=True)

    class Meta:
        abstract = True


class ContactModelMixin(models.Model):
    email = models.EmailField(default="", blank=True)

    email_alternative = models.EmailField(default="", blank=True)

    telephone = models.CharField(max_length=15, default="", blank=True)

    telephone_alternative = models.CharField(max_length=15, default="", blank=True)

    class Meta:
        abstract = True



from django.contrib import admin

from edc_data_manager.admin_site import edc_data_manager_admin
from edc_form_runners.admin import IssueAdmin as BaseIssueAdmin

from ..models import Issue


@admin.register(Issue, site=edc_data_manager_admin)
class IssueAdmin(BaseIssueAdmin):
    pass



# Generated by Django 4.1 on 2022-08-21 18:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_offstudy", "0011_auto_20220714_2136"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalsubjectoffstudy",
            old_name="offstudy_reason_other",
            new_name="other_offstudy_reason",
        ),
        migrations.RenameField(
            model_name="subjectoffstudy",
            old_name="offstudy_reason_other",
            new_name="other_offstudy_reason",
        ),
        migrations.AddField(
            model_name="historicalsubjectoffstudy",
            name="comment",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="Please provide further details if possible",
            ),
        ),
        migrations.AddField(
            model_name="subjectoffstudy",
            name="comment",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="Please provide further details if possible",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectoffstudy",
            name="offstudy_reason",
            field=models.CharField(
                choices=[
                    ("lost", "Lost to follow-up"),
                    ("completed protocol", "Completed protocol"),
                    ("consent_withdrawn", "Consent withdrawn"),
                    ("dead", "Deceased"),
                    ("OTHER", "Other, please specify ..."),
                ],
                max_length=125,
                verbose_name="Please code the primary reason participant taken off-study",
            ),
        ),
        migrations.AlterField(
            model_name="subjectoffstudy",
            name="offstudy_reason",
            field=models.CharField(
                choices=[
                    ("lost", "Lost to follow-up"),
                    ("completed protocol", "Completed protocol"),
                    ("consent_withdrawn", "Consent withdrawn"),
                    ("dead", "Deceased"),
                    ("OTHER", "Other, please specify ..."),
                ],
                max_length=125,
                verbose_name="Please code the primary reason participant taken off-study",
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2024-11-09 22:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_pylabels", "0006_delete_drawing"),
    ]

    operations = [
        migrations.AlterField(
            model_name="labelconfiguration",
            name="name",
            field=models.CharField(
                help_text="Name of configuration registered with site_label_config.",
                max_length=50,
                unique=True,
            ),
        ),
    ]



from django.db import models

from edc_model.models import BaseUuidModel


class GradingException(BaseUuidModel):

    reference_range_collection = models.ForeignKey(
        "edc_reportable.ReferenceRangeCollection", on_delete=models.PROTECT
    )

    label = models.CharField(max_length=50)

    grade1 = models.BooleanField(default=False)
    grade2 = models.BooleanField(default=False)
    grade3 = models.BooleanField(default=False)
    grade4 = models.BooleanField(default=False)

    def grades(self) -> list[int]:
        grades = []
        for i in range(1, 5):
            if getattr(self, f"grade{i}"):
                grades.append(i)
        return grades

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Grading Exception"
        verbose_name_plural = "Grading Exceptions"



from .ae_followup_modelform_mixin import AeFollowupModelFormMixin
from .ae_initial_modelform_mixin import AeInitialModelFormMixin
from .ae_modelform_mixin import AeModelFormMixin
from .ae_susar_modelform_mixin import AeSusarModelFormMixin
from .ae_tmg_modelform_mixin import AeTmgModelFormMixin
from .death_report_modelform_mixin import DeathReportModelFormMixin
from .death_report_tmg_modelform_mixin import DeathReportTmgModelFormMixin
from .death_report_tmg_second_modelform_mixin import DeathReportTmgSecondModelFormMixin
from .hospitalization_modelform_mixin import HospitalizationModelFormMixin



from __future__ import annotations

from django.db import models

from ..model_mixin_factories import glucose_model_mixin_factory
from .fasting_model_mixin import fasting_model_mixin_factory


class GlucoseModelMixin(
    glucose_model_mixin_factory("glucose"),
    fasting_model_mixin_factory("glucose"),
    models.Model,
):
    """A model mixin of fields for the Glucose, fasting or random"""

    class Meta:
        abstract = True



from django.db import models

from edc_list_data.model_mixins import ListModelMixin


class ContainerUnits(ListModelMixin):
    """Container Units Model

    For example, tablets, ml, etc
    """

    display_name = models.CharField(
        verbose_name="Name",
        max_length=250,
        unique=True,
        null=True,
        blank=True,
        help_text="(suggest 40 characters max.)",
    )

    class Meta(ListModelMixin.Meta):
        verbose_name = "Container units"
        verbose_name_plural = "Container units"



from .activate_refill import activate_refill
from .adjust_previous_end_datetime import adjust_previous_end_datetime
from .create_next_refill import create_next_refill
from .create_refill import create_refill
from .crf import create_refills_from_crf, delete_next_refills_for_crf
from .deactivate_refill import deactivate_refill
from .get_active_refill import get_active_refill
from .refill_creator import RefillCreator



from edc_lab.models import Panel


class RequisitionPanel(Panel):
    class Meta:
        proxy = True
        default_permissions = ("view", "export")
        verbose_name = "Requisition Panel"
        verbose_name_plural = "Requisition Panels"



from edc_lab.models import BoxItem
from edc_listboard.filters import ListboardFilter, ListboardViewFilters


def get_box_items():
    return BoxItem.objects.all().order_by("position").values("identifier")


class AliquotListboardViewFilters(ListboardViewFilters):
    all = ListboardFilter(name="all", label="All", lookup={})

    is_primary = ListboardFilter(label="Primary", lookup={"is_primary": True})

    packed = ListboardFilter(label="Packed", lookup={"aliquot_identifier__in": get_box_items})

    not_packed = ListboardFilter(
        label="Not Packed",
        exclude_filter=True,
        lookup={"aliquot_identifier__in": get_box_items},
    )

    shipped = ListboardFilter(label="Shipped", lookup={"shipped": True})

    not_shipped = ListboardFilter(label="Not shipped", default=True, lookup={"shipped": False})



from edc_model_wrapper import ModelWrapper

from edc_lab.models import Box, ManifestItem


class ManifestItemModelWrapper(ModelWrapper):
    model_cls = ManifestItem
    next_url_name = "manage_manifest_listboard_url"
    action_name = "manage"

    @property
    def manifest_identifier(self):
        return self.object.manifest.manifest_identifier

    @property
    def box_identifier(self):
        return self.object.identifier

    @property
    def box(self):
        return Box.objects.get(box_identifier=self.object.identifier)



from typing import Any

from django.db import models
from django.utils import timezone

from edc_sites.model_mixins import SiteModelMixin

from ...choices import (
    ALIQUOT_CONDITIONS,
    ALIQUOT_STATUS,
    SPECIMEN_MEASURE_UNITS,
    SPECIMEN_MEDIUM,
)


class AliquotModelMixin(SiteModelMixin, models.Model):
    aliquot_datetime = models.DateTimeField(
        verbose_name="Date and time aliquot created", default=timezone.now
    )

    is_primary = models.BooleanField(default=False, editable=False)

    count = models.IntegerField(
        editable=False, help_text="pos in sequence of aliquots from parent"
    )

    # FIXME: not used
    medium = models.CharField(
        verbose_name="Medium", max_length=25, choices=SPECIMEN_MEDIUM, default="TUBE"
    )

    # FIXME: not used
    medium_count = models.IntegerField(
        editable=False, default=1, help_text="e.g. number of tubes"
    )

    original_measure = models.DecimalField(max_digits=10, decimal_places=2, default="5.00")

    current_measure = models.DecimalField(max_digits=10, decimal_places=2, default="5.00")

    measure_units = models.CharField(
        max_length=25, choices=SPECIMEN_MEASURE_UNITS, default="mL"
    )

    status = models.CharField(max_length=25, choices=ALIQUOT_STATUS, default="available")

    condition = models.CharField(max_length=25, choices=ALIQUOT_CONDITIONS, default="10")

    comment = models.CharField(max_length=50, blank=True, default="")

    def __str__(self: Any):
        return self.aliquot_identifier

    def natural_key(self: Any):
        return (self.aliquot_identifier,)

    natural_key.dependencies = ("sites.Site",)

    class Meta:
        abstract = True



from copy import copy

from django.db.models import QuerySet
from django.utils.translation import gettext_lazy as _

from edc_constants.constants import DWTA, NOT_APPLICABLE, NULL_STRING

from .base_form_validator import (
    NOT_REQUIRED_ERROR,
    REQUIRED_ERROR,
    BaseFormValidator,
    InvalidModelFormFieldValidator,
)


class RequiredFieldValidatorError(Exception):
    pass


class RequiredFieldValidator(BaseFormValidator):
    def raise_required(
        self, field: str, msg: str | None = None, inline_set: str | None = None
    ) -> None:
        if inline_set:
            default_errmsg = _("Based on your responses, inline information is required.")
            message = {"__all__": (msg or default_errmsg).strip()}
        else:
            errmsg = _("This field is required")
            message = {field: f"{errmsg}. {msg or ''}".strip()}
        self.raise_validation_error(message, REQUIRED_ERROR)

    def raise_not_required(
        self, field: str, msg: str | None = None, inline_set: str | None = None
    ) -> None:
        if inline_set:
            default_errmsg = _("Based on your responses, inline information is not required.")
            message = {"__all__": (msg or default_errmsg).strip()}
        else:
            errmsg = _("This field is not required")
            message = {field: f"{errmsg}. {msg or ''}".strip()}
        self.raise_validation_error(message, NOT_REQUIRED_ERROR)

    def required_if(
        self,
        *responses: str | int | bool,
        field: str,
        field_required: str,
        required_msg: str | None = None,
        not_required_msg: str | None = None,
        optional_if_dwta: bool | None = None,
        optional_if_na: bool | None = None,
        inverse: bool | None = None,
        is_instance_field: bool | None = None,
        field_required_evaluate_as_int: bool | None = None,
        fk_stored_field_name=None,
        field_required_inline_set=None,
    ) -> bool:
        """Raises an exception or returns False.

        if field in responses then field_required is required.

        is_instance_field: value comes from the model instance and not cleaned data
        """
        inverse = True if inverse is None else inverse
        inline_set = field_required_inline_set
        if is_instance_field:
            self.update_cleaned_data_from_instance(field)
        responses = self._convert_response_to_values_if_instances(
            responses, fk_stored_field_name
        )
        self._inspect_params(*responses, field=field, field_required=field_required)
        field_value = self.get(field)

        if isinstance(field_value, (QuerySet,)):
            raise RequiredFieldValidatorError(
                "Field value is a QuerySet, is this an M2M? "
                f"Field='{field}'. See {self.__class__.__name__}."
            )

        if field_required_evaluate_as_int:
            field_required_has_value = (
                self.get(field_required, inline_set=inline_set) is not None
            )
        else:
            field_required_has_value = bool(self.get(field_required, inline_set=inline_set))

        if field in self.cleaned_data:
            if (DWTA in responses and optional_if_dwta and field_value == DWTA) or (
                NOT_APPLICABLE in responses
                and optional_if_na
                and field_value == NOT_APPLICABLE
            ):
                pass
            elif field_value in responses and (
                not field_required_has_value
                or self.get(field_required, inline_set=inline_set) == NOT_APPLICABLE
            ):
                self.raise_required(
                    field=field_required,
                    msg=required_msg,
                    inline_set=inline_set,
                )
            elif inverse and (
                field_value not in responses
                and (
                    field_required_has_value
                    and (self.get(field_required, inline_set=inline_set) != NOT_APPLICABLE)
                )
            ):
                self.raise_not_required(
                    field=field_required,
                    msg=not_required_msg,
                    inline_set=inline_set,
                )
        return False

    def required_if_true(
        self,
        condition: bool,
        field_required: str,
        required_msg: str | None = None,
        not_required_msg: str | None = None,
        inverse: bool | None = None,
    ) -> bool:
        inverse = True if inverse is None else inverse
        if not field_required:
            errmsg = _("The required field cannot be None.")
            raise InvalidModelFormFieldValidator(errmsg)
        if self.cleaned_data and field_required in self.cleaned_data:
            if condition and (
                self.cleaned_data.get(field_required) in [None, NULL_STRING, NOT_APPLICABLE]
            ):
                self.raise_required(field=field_required, msg=required_msg)
            elif inverse and (
                not condition
                and self.cleaned_data.get(field_required)
                not in [None, NULL_STRING, NOT_APPLICABLE]
            ):
                self.raise_not_required(field=field_required, msg=not_required_msg)
        return False

    def not_required_if_true(
        self,
        condition: bool,
        field: str,
        msg: str | None = None,
        is_instance_field: bool | None = None,
    ) -> bool:
        """Raises a ValidationError if condition is True stating the
        field is NOT required.

        The inverse is not tested.
        """
        if not field:
            errmsg = _("The required field cannot be None.")
            raise InvalidModelFormFieldValidator(errmsg)
        if is_instance_field:
            self.update_cleaned_data_from_instance(field)
        if self.cleaned_data and field in self.cleaned_data:
            try:
                field_value = self.cleaned_data.get(field).name
            except AttributeError:
                field_value = self.cleaned_data.get(field)
            if condition and field_value is not None and field_value != NOT_APPLICABLE:
                self.raise_not_required(field=field, msg=msg)
        return False

    def required_if_not_none(
        self,
        field: str,
        field_required: str,
        required_msg: str | None = None,
        not_required_msg: str | None = None,
        optional_if_dwta: bool | None = None,
        inverse: bool | None = None,
        field_required_evaluate_as_int: bool | None = None,
        is_instance_field: bool | None = None,
    ) -> bool:
        """Raises an exception or returns False.

        If field is not none, field_required is "required".

        Note: CharFields usually default to an empty string and not NULL.
           For IntegerFields, zero is a value.
        """
        inverse = True if inverse is None else inverse
        if is_instance_field:
            self.update_cleaned_data_from_instance(field)
        if not field_required:
            errmsg = _("The required field cannot be None.")
            raise InvalidModelFormFieldValidator(errmsg)
        if optional_if_dwta and self.cleaned_data.get(field) == DWTA:
            field_value = None
        else:
            field_value = self.cleaned_data.get(field)

        if field_required_evaluate_as_int:
            field_required_has_value = self.cleaned_data.get(field_required) not in [
                None,
                NULL_STRING,
            ]
        else:
            field_required_has_value = bool(self.cleaned_data.get(field_required))

        if field_value not in [None, NULL_STRING] and not field_required_has_value:
            self.raise_required(field=field_required, msg=required_msg)
        elif (
            field_value in [None, NULL_STRING]
            and field_required_has_value
            and self.cleaned_data.get(field_required) != NOT_APPLICABLE
            and inverse
        ):
            self.raise_not_required(field=field_required, msg=not_required_msg)
        return False

    def required_integer_if_not_none(self, **kwargs):
        """Raises an exception or returns False.

        Evaluates the value of field required as an integer, that is,
        0 is not None.
        """
        return self.required_if_not_none(field_required_evaluate_as_int=True, **kwargs)

    def not_required_if(
        self,
        *responses: str | int | bool,
        field: str,
        field_required: str | None = None,
        field_not_required: str | None = None,
        required_msg: str | None = None,
        not_required_msg: str | None = None,
        optional_if_dwta: bool | None = None,
        inverse: bool | None = None,
        is_instance_field: bool | None = None,
    ) -> bool:
        """Raises an exception or returns False.

        if field NOT in responses then field_required is required.
        """
        inverse = True if inverse is None else inverse
        field_required = field_required or field_not_required
        if is_instance_field:
            self.update_cleaned_data_from_instance(field)
        self._inspect_params(*responses, field=field, field_required=field_required)
        if field in self.cleaned_data and field_required in self.cleaned_data:
            if DWTA in responses and optional_if_dwta and self.cleaned_data.get(field) == DWTA:
                pass
            elif self.cleaned_data.get(field) in responses and (
                self.cleaned_data.get(field_required)
                and self.cleaned_data.get(field_required) != NOT_APPLICABLE
            ):
                self.raise_not_required(field=field_required, msg=not_required_msg)
            elif inverse and (
                self.cleaned_data.get(field) not in responses
                and (
                    not self.cleaned_data.get(field_required)
                    or (self.cleaned_data.get(field_required) == NOT_APPLICABLE)
                )
            ):
                self.raise_required(field=field_required, msg=required_msg)
        return False

    def require_together(
        self,
        field: str,
        field_required: str,
        required_msg: str | None = None,
        is_instance_field: bool | None = None,
    ) -> bool:
        """Required `b` if `a`; do not require `b` if not `a`"""
        if is_instance_field:
            self.update_cleaned_data_from_instance(field)
        if self.cleaned_data.get(field) is not None and self.cleaned_data.get(
            field_required
        ) in [None, NULL_STRING]:
            self.raise_required(field=field_required, msg=required_msg)
        elif (
            self.cleaned_data.get(field) in [None, NULL_STRING]
            and self.cleaned_data.get(field_required) is not None
        ):
            self.raise_not_required(field=field_required, msg=required_msg)
        return False

    @staticmethod
    def _inspect_params(*responses: str | int | bool, field: str, field_required: str) -> bool:
        """Inspects params and raises if any are None"""
        if not field:
            errmsg = _("`field` cannot be `None`")
            raise InvalidModelFormFieldValidator(f"{errmsg}.")
        if not responses:
            errmsg = _(
                "At least one valid response for field '{field}' must be provided."
            ).format(field=field)
            raise InvalidModelFormFieldValidator(errmsg)
        if not field_required:
            raise InvalidModelFormFieldValidator('"field_required" cannot be None.')
        return False

    @staticmethod
    def _convert_response_to_values_if_instances(responses, fk_stored_field_name):
        fk_stored_field_name = fk_stored_field_name or "name"
        responses = list(responses)
        _responses = copy(responses)
        for index, response in enumerate(_responses):
            responses[index] = getattr(response, fk_stored_field_name, response)
        return tuple(responses)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.utils.html import format_html
from django.utils.safestring import mark_safe

from edc_constants.constants import NO, PENDING, TBD, YES

from .exceptions import (
    ScreeningEligibilityAttributeError,
    ScreeningEligibilityError,
    ScreeningEligibilityInvalidCombination,
    ScreeningEligibilityModelAttributeError,
)
from .fc import FC

if TYPE_CHECKING:
    from edc_model.models import BaseUuidModel

    from .model_mixins import EligibilityModelMixin, ScreeningModelMixin

    class SubjectScreeningModel(ScreeningModelMixin, BaseUuidModel): ...


__all__ = ["ScreeningEligibility"]


class ScreeningEligibility:
    """A class to calculate eligibility criteria."""

    eligible_display_label: str = "ELIGIBLE"
    eligible_fld_name: str = "eligible"
    eligible_value_default: str = TBD
    default_display_label = TBD
    eligible_values_list: list = [YES, NO, TBD]
    ineligible_display_label: str = "INELIGIBLE"
    is_eligible_value: str = YES
    is_ineligible_value: str = NO
    pending_value = PENDING
    pending_display_label = "PENDING"
    reasons_ineligible_fld_name: str = "reasons_ineligible"

    def __init__(
        self,
        model_obj: SubjectScreeningModel | EligibilityModelMixin = None,
        cleaned_data: dict = None,
        eligible_value_default: str | None = None,
        eligible_values_list: list | None = None,
        is_eligible_value: str | None = None,
        is_ineligible_value: str | None = None,
        eligible_display_label: str | None = None,
        ineligible_display_label: str | None = None,
        update_model: bool | None = None,
    ) -> None:
        self.eligible: str = ""
        self.reasons_ineligible: dict[str, str] = {}
        self.model_obj = model_obj
        self.update_model: bool = True if update_model is None else update_model
        self.cleaned_data = cleaned_data
        if eligible_value_default:
            self.eligible_value_default = eligible_value_default
        if eligible_values_list:
            self.eligible_values_list = eligible_values_list
        if is_eligible_value:
            self.is_eligible_value = is_eligible_value
        if is_ineligible_value:
            self.is_ineligible_value = is_ineligible_value
        if eligible_display_label:
            self.eligible_display_label = eligible_display_label
        if ineligible_display_label:
            self.ineligible_display_label = ineligible_display_label

        self._assess_eligibility()

        if self.eligible not in self.eligible_values_list:
            raise ScreeningEligibilityError(
                f"Invalid value. See attr `eligible`. Expected one of "
                f"{self.eligible_values_list}. Got {self.eligible}."
            )
        if self.eligible == self.is_eligible_value and self.reasons_ineligible:
            raise ScreeningEligibilityInvalidCombination(
                "Inconsistent result. Got eligible==YES where reasons_ineligible"
                f"is not None. Got reasons_ineligible={self.reasons_ineligible}"
            )
        if self.eligible == self.is_ineligible_value and not self.reasons_ineligible:
            raise ScreeningEligibilityInvalidCombination(
                f"Inconsistent result. Got eligible=={self.eligible} "
                "where reasons_ineligible is None"
            )
        if self.model_obj and self.update_model:
            self._set_fld_attrs_on_model()

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}()"

    def __str__(self) -> str:
        return f"{self.__class__.__name__}()"

    def assess_eligibility(self) -> None:
        """Override to add additional assessments after the default
        assessment is complete.

        Will only run if the default assessment returns `is eligible`
        """
        pass

    def get_required_fields(self) -> dict[str, FC | None]:
        """Returns a dict of {field_name: FC(value, msg), ...} needed
        to determine eligibility.

        * dict `key` is the field name. Should correspond with the model
          field name.
        * dict `value` is an `FC` instance.

        """
        return {}

    def set_fld_attrs_on_model(self) -> None:
        """Override to update additional model fields.

        Called after `assess_eligibity`.
        """
        pass

    @property
    def is_eligible(self) -> bool:
        """Returns True if eligible else False"""
        return True if self.eligible == self.is_eligible_value else False

    def _assess_eligibility(self) -> None:
        self.set_fld_attrs_on_self()
        self.eligible = self.is_eligible_value
        missing_data = self.get_missing_data()
        if missing_data:
            self.reasons_ineligible.update(**missing_data)
            self.eligible = self.eligible_value_default  # probably TBD
        for fldattr, fc in self.get_required_fields().items():
            if fldattr not in missing_data:
                if fc and fc.value:
                    msg = fc.msg if fc.msg else fldattr.title().replace("_", " ")
                    is_callable = False
                    try:
                        value = fc.value(getattr(self, fldattr))
                    except TypeError:
                        value = fc.value
                    else:
                        is_callable = True
                    if (
                        (isinstance(value, str) and getattr(self, fldattr) != value)
                        or (
                            isinstance(value, (list, tuple))
                            and getattr(self, fldattr) not in value
                        )
                        or (
                            isinstance(value, range)
                            and not (min(value) <= getattr(self, fldattr) <= max(value))
                        )
                        or (is_callable and value is False)
                    ):
                        self.reasons_ineligible.update({fldattr: msg})
                        self.eligible = self.is_ineligible_value  # probably NO
        if self.is_eligible:
            if self.is_eligible and not self.get_required_fields():
                self.eligible = self.eligible_value_default
            self.assess_eligibility()

    def _set_fld_attrs_on_model(self) -> None:
        """Updates screening model's eligibility field values.

        Called in the model.save() method so no need to call
        model.save().
        """
        setattr(
            self.model_obj,
            self.reasons_ineligible_fld_name,
            "|".join(self.reasons_ineligible.values()) or "",
        )
        self.set_eligible_model_field()
        self.set_fld_attrs_on_model()

    def set_eligible_model_field(self):
        setattr(self.model_obj, self.eligible_fld_name, self.is_eligible)

    def set_fld_attrs_on_self(self) -> None:
        """Adds fld attrs from the model / cleaned_data to self"""

        for fldattr in self.get_required_fields():
            try:
                getattr(self, fldattr)
            except AttributeError as e:
                raise ScreeningEligibilityAttributeError(
                    "Attribute refered to in `required_fields` "
                    "does not exist on class. "
                    f"See {self.__class__.__name__}. "
                    f"Got {e}"
                )
            if self.model_obj:
                try:
                    value = (
                        self.cleaned_data.get(fldattr)
                        if self.cleaned_data
                        else getattr(self.model_obj, fldattr)
                    )
                except AttributeError as e:
                    raise ScreeningEligibilityModelAttributeError(
                        "Attribute does not exist on model. "
                        f"See {self.model_obj.__class__.__name__}. "
                        f"Got {e}"
                    )
            else:
                value = self.cleaned_data.get(fldattr)
            setattr(self, fldattr, value)

    def get_missing_data(self) -> dict:
        missing_responses = {}
        for fldattr, fc in self.get_required_fields().items():
            if fc and not fc.ignore_if_missing:
                value = getattr(self, fldattr)
                if value:
                    if fc.missing_value and value == fc.missing_value:
                        missing_responses.update({fldattr: None})
                else:
                    missing_responses.update({fldattr: value})
        return {
            k: f"`{k.replace('_', ' ').title()}` not answered"
            for k, v in missing_responses.items()
            if not v
        }

    def formatted_reasons_ineligible(self) -> str:
        str_values = "<BR>".join([x for x in self.reasons_ineligible.values() if x])
        return format_html(
            "{}",
            mark_safe(str_values),  # nosec B703 B308
        )

    @property
    def display_label(self) -> str:
        display_label = self.eligible
        if self.eligible == self.is_eligible_value:
            display_label = self.eligible_display_label
        elif self.eligible == self.is_ineligible_value:
            display_label = self.ineligible_display_label
        elif self.eligible == self.eligible_value_default:
            display_label = self.default_display_label
        elif self.eligible == self.pending_value:
            display_label = self.pending_display_label
        return display_label



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from django_audit_fields.admin import audit_fieldset_tuple

from edc_model_admin.history import SimpleHistoryAdmin
from edc_utils.date import to_local

from ...admin_site import edc_pharmacy_admin
from ...forms import ReceiveItemForm
from ...models import OrderItem, Receive, ReceiveItem
from ...utils import format_qty
from ..actions import delete_receive_items_action, print_labels_from_receive_item
from ..model_admin_mixin import ModelAdminMixin


@admin.register(ReceiveItem, site=edc_pharmacy_admin)
class ReceiveItemAdmin(ModelAdminMixin, SimpleHistoryAdmin):
    change_list_title = "Pharmacy: Received items"
    change_form_title = "Pharmacy: Receive item"
    history_list_display = ()
    show_object_tools = False
    show_cancel = True
    list_per_page = 20

    form = ReceiveItemForm
    include_audit_fields_in_list_display = False
    ordering = ("-receive_item_identifier",)

    actions = [delete_receive_items_action, print_labels_from_receive_item]

    fieldsets = (
        (
            None,
            {"fields": ("receive", "order_item", "container", "lot")},
        ),
        (
            "Quantity",
            {"fields": ("qty", "unit_qty")},
        ),
        (
            "Reference",
            {"fields": ("reference", "comment")},
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "identifier",
        "item_date",
        "order_item_product",
        "formatted_lot",
        "container",
        "formatted_qty",
        "formatted_unit_qty",
        "order_changelist",
        "order_items_changelist",
        "receive_changelist",
        "stock_changelist",
        "reference",
        "modified",
        "user_created",
        "user_modified",
    )
    list_filter = (
        "receive_item_datetime",
        "lot",
        "reference",
        "created",
        "modified",
    )
    search_fields = (
        "id",
        "order_item__id",
        "order_item__order__order_identifier",
        "receive__id",
        "container__name",
        "lot__lot_no",
        "reference",
        "comment",
    )

    readonly_fields = ("unit_qty",)

    @admin.display(description="Item date", ordering="receive_item_datetime")
    def item_date(self, obj):
        return to_local(obj.receive_item_datetime).date()

    @admin.display(description="BATCH #", ordering="lot__lot_no")
    def formatted_lot(self, obj):
        return obj.lot.lot_no

    @admin.display(description="QTY", ordering="qty")
    def formatted_qty(self, obj):
        return format_qty(obj.qty, obj.container)

    @admin.display(description="Units", ordering="qty")
    def formatted_unit_qty(self, obj):
        return format_qty(obj.unit_qty, obj.container)

    @admin.display(description="Product", ordering="-order_item__product__name")
    def order_item_product(self, obj):
        return obj.order_item.product

    @admin.display(description="Receive #", ordering="-receive__receive_datetime")
    def receive_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_receive_changelist")
        url = f"{url}?q={obj.receive.receive_identifier}"
        context = dict(
            url=url, label=obj.receive.receive_identifier, title="Back to receiving"
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="Stock")
    def stock_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_stock_changelist")
        url = f"{url}?q={obj.id}"
        label = "Stock"
        context = dict(url=url, label=label, title="Go to stock")
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="Order #")
    def order_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_order_changelist")
        url = f"{url}?q={obj.order_item.order.order_identifier!s}"
        context = dict(
            url=url, label=obj.order_item.order.order_identifier, title="Back to order"
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="Order item #")
    def order_items_changelist(self, obj):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_orderitem_changelist")
        url = f"{url}?q={obj.order_item.id!s}"
        context = dict(
            url=url,
            label=obj.order_item.order_item_identifier,
            title="Back to order item",
        )
        return render_to_string("edc_pharmacy/stock/items_as_link.html", context=context)

    @admin.display(description="RECEIVE ITEM #", ordering="-receive_item_identifier")
    def identifier(self, obj):
        return obj.receive_item_identifier

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ("receive",)
        return self.readonly_fields

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "receive":
            if request.GET.get("receive"):
                kwargs["queryset"] = Receive.objects.filter(
                    id__exact=request.GET.get("receive", 0)
                )
            else:
                kwargs["queryset"] = Receive.objects.none()
        if db_field.name == "order_item":
            if request.GET.get("order_item"):
                kwargs["queryset"] = OrderItem.objects.filter(
                    id__exact=request.GET.get("order_item", 0)
                )
            else:
                kwargs["queryset"] = OrderItem.objects.none()
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



from edc_adverse_event.constants import AE_WITHDRAWN
from edc_constants.constants import YES
from edc_form_validators import INVALID_ERROR, FormValidator
from edc_reportable import SEVERITY_INCREASED_FROM_G3


class AeFollowupFormValidator(FormValidator):
    def clean(self):
        self.applicable_if(
            SEVERITY_INCREASED_FROM_G3, field="outcome", field_applicable="ae_grade"
        )
        self.applicable_if(
            SEVERITY_INCREASED_FROM_G3, field="outcome", field_applicable="ae_grade"
        )
        if (
            self.cleaned_data.get("outcome")
            and self.cleaned_data.get("followup")
            and self.cleaned_data.get("outcome") == AE_WITHDRAWN
            and self.cleaned_data.get("followup") == YES
        ):
            self.raise_validation_error(
                {"followup": "AE was reported as withrawn. Expected NO"}, INVALID_ERROR
            )



import math
from decimal import Decimal
from uuid import uuid4

from django.conf import settings
from django.db import models
from django.db.models.deletion import PROTECT

from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import convert_php_dateformat
from edc_utils.round_up import round_half_away_from_zero

from ...dosage_calculator import DosageCalculator
from ...model_mixins import MedicationOrderModelMixin, PreviousNextModelMixin
from ..medication import DosageGuideline, Formulation
from .rx import Rx


class Manager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, prescription, medication, refill_start_datetime):
        return self.get(prescription, medication, refill_start_datetime)


class RxRefill(
    PreviousNextModelMixin,
    MedicationOrderModelMixin,
    SiteModelMixin,
    BaseUuidModel,
):
    rx = models.ForeignKey(Rx, on_delete=PROTECT)

    refill_identifier = models.CharField(max_length=36, default=uuid4, editable=False)

    dosage_guideline = models.ForeignKey(DosageGuideline, on_delete=PROTECT)

    formulation = models.ForeignKey(Formulation, on_delete=PROTECT, null=True)

    dose = models.DecimalField(
        max_digits=6,
        decimal_places=1,
        null=True,
        blank=True,
        help_text="dose per frequency if NOT considering weight",
    )

    weight_in_kgs = models.DecimalField(
        max_digits=6,
        decimal_places=1,
        null=True,
        blank=True,
        help_text="Defaults to 1.0",
    )

    refill_start_datetime = models.DateTimeField(
        verbose_name="Refill start date/time",
        help_text="Starting date for this refill",
    )

    refill_end_datetime = models.DateTimeField(
        verbose_name="Refill end date/time",
        null=True,
        help_text="Ending date for this refill",
    )

    number_of_days = models.IntegerField(null=True)

    roundup_dose = models.BooleanField(
        default=False, help_text="Rounds UP the dose. e.g. 7.3->8.0, 7.5->8.0"
    )

    round_dose = models.IntegerField(
        default=0, help_text="Rounds the dose. e.g. 7.3->7.0, 7.5->8.0"
    )

    roundup_divisible_by = models.IntegerField(
        default=0,
        help_text="Rounds up the total. For example, 32 would round 112 pills to 128 pills",
    )

    calculate_dose = models.BooleanField(default=True)

    total = models.DecimalField(
        max_digits=10,
        decimal_places=4,
        null=True,
        blank=True,
        help_text="Total to be dispensed. Leave blank to auto-calculate",
    )

    remaining = models.DecimalField(
        max_digits=6,
        decimal_places=1,
        null=True,
        blank=True,
        help_text="Leave blank to auto-calculate",
    )

    notes = models.TextField(
        max_length=250,
        default="",
        blank=True,
        help_text="Additional information for patient",
    )

    active = models.BooleanField(default=False)

    verified = models.BooleanField(default=False)

    verified_datetime = models.DateTimeField(null=True, blank=True)

    as_string = models.CharField(max_length=150, editable=False)

    objects = Manager()

    on_site = CurrentSiteManager()

    history = HistoricalRecords()

    def __str__(self):
        convert_php_dateformat(settings.SHORT_DATE_FORMAT)
        start_date = self.refill_start_datetime.strftime(
            convert_php_dateformat(settings.SHORT_DATE_FORMAT)
        )
        end_date = self.refill_end_datetime.strftime(
            convert_php_dateformat(settings.SHORT_DATE_FORMAT)
        )
        return (
            f"{self.rx} "
            f"Take {self.dose} {self.formulation.formulation_type.display_name} "
            f"{self.formulation.route.display_name}. "
            f"Valid: {start_date} to {end_date}."
        )

    def natural_key(self):
        return (self.refill_identifier,)

    def save(self, *args, **kwargs):
        self.adjust_end_datetimes()
        self.number_of_days = (self.refill_end_datetime - self.refill_start_datetime).days
        if not self.weight_in_kgs:
            self.weight_in_kgs = self.rx.weight_in_kgs
        self.dose = Decimal(str(self.get_dose()))
        self.total = Decimal(str(self.get_total_to_dispense()))
        if not self.id:
            self.remaining = self.total
        self.as_string = str(self)
        super().save(*args, **kwargs)

    @property
    def frequency(self):
        return self.dosage_guideline.frequency

    @property
    def frequency_units(self):
        return self.dosage_guideline.frequency_units

    def get_dose(self) -> float:
        dosage = DosageCalculator(
            dosage_guideline=self.dosage_guideline,
            formulation=self.formulation,
            weight_in_kgs=self.weight_in_kgs,
        ).dosage
        if self.roundup_dose:
            return math.ceil(dosage)
        if self.round_dose:
            return round_half_away_from_zero(dosage, self.round_dose)
        return dosage

    def get_total_to_dispense(self):
        """Returns total 'dispense unit (e.g.pills)' rounded up if
        divisor is greater than 1.
        """
        if self.roundup_divisible_by:
            return (
                math.ceil(
                    (
                        float(self.get_dose())
                        * float(self.frequency)
                        * float(self.number_of_days)
                    )
                    / self.roundup_divisible_by
                )
                * self.roundup_divisible_by
            )
        return float(self.get_dose()) * float(self.frequency) * float(self.number_of_days)

    @property
    def subject_identifier(self):
        return self.rx.subject_identifier

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Prescription: Refill"
        verbose_name_plural = "Prescription: Refills"



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_adverse_event",
            "0013_alter_aeactionclassification_device_created_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="aeactionclassification",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="aeclassification",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="causeofdeath",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="edcpermissions",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Edc Permissions",
                "verbose_name_plural": "Edc Permissions",
            },
        ),
        migrations.AlterModelOptions(
            name="saereason",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.RemoveIndex(
            model_name="aeactionclassification",
            name="edc_adverse_id_5fd6ee_idx",
        ),
        migrations.RemoveIndex(
            model_name="aeclassification",
            name="edc_adverse_id_6431b6_idx",
        ),
        migrations.RemoveIndex(
            model_name="causeofdeath",
            name="edc_adverse_id_ca88f1_idx",
        ),
        migrations.RemoveIndex(
            model_name="saereason",
            name="edc_adverse_id_dc0e62_idx",
        ),
        migrations.AddField(
            model_name="aeactionclassification",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aeactionclassification",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aeclassification",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aeclassification",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="causeofdeath",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="causeofdeath",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="edcpermissions",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="edcpermissions",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="saereason",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="saereason",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="aeactionclassification",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="aeactionclassification",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="aeactionclassification",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="aeclassification",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="aeclassification",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="aeclassification",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="causeofdeath",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="causeofdeath",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="causeofdeath",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AlterField(
            model_name="saereason",
            name="display_index",
            field=models.IntegerField(
                default=0,
                help_text="Index to control display order if not alphabetical, not required",
                verbose_name="display index",
            ),
        ),
        migrations.AlterField(
            model_name="saereason",
            name="display_name",
            field=models.CharField(
                help_text="(suggest 40 characters max.)",
                max_length=250,
                unique=True,
                verbose_name="Name",
            ),
        ),
        migrations.AlterField(
            model_name="saereason",
            name="name",
            field=models.CharField(
                help_text="This is the stored value, required",
                max_length=250,
                unique=True,
                verbose_name="Stored value",
            ),
        ),
        migrations.AddIndex(
            model_name="aeactionclassification",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_adverse_display_9f6c78_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aeactionclassification",
            index=models.Index(
                fields=["modified", "created"], name="edc_adverse_modifie_2d407c_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="aeactionclassification",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_adverse_user_mo_b0d69e_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aeclassification",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_adverse_display_c2c113_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="aeclassification",
            index=models.Index(
                fields=["modified", "created"], name="edc_adverse_modifie_970ccb_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="aeclassification",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_adverse_user_mo_969781_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="causeofdeath",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_adverse_display_4c7e3b_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="causeofdeath",
            index=models.Index(
                fields=["modified", "created"], name="edc_adverse_modifie_835e5b_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="causeofdeath",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_adverse_user_mo_e41e6f_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="edcpermissions",
            index=models.Index(
                fields=["modified", "created"], name="edc_adverse_modifie_69a2bc_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="edcpermissions",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_adverse_user_mo_7f4957_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="saereason",
            index=models.Index(
                fields=["display_index", "display_name"],
                name="edc_adverse_display_2a23e9_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="saereason",
            index=models.Index(
                fields=["modified", "created"], name="edc_adverse_modifie_439471_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="saereason",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_adverse_user_mo_09db8b_idx",
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2017-09-21 07:19
from __future__ import unicode_literals

import _socket
import django_audit_fields.fields.uuid_auto_field
import django_revision.revision_field
from django.db import migrations, models

import edc_model_fields.fields.hostname_modification_field
import edc_model_fields.fields.userfield
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [("edc_lab", "0007_auto_20170321_1119")]

    operations = [
        migrations.CreateModel(
            name="IdentifierHistory",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("identifier", models.CharField(max_length=50, unique=True)),
                ("identifier_type", models.CharField(max_length=50)),
                ("identifier_prefix", models.CharField(max_length=25, null=True)),
            ],
            options={"abstract": False},
        ),
        migrations.AddField(
            model_name="aliquot",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="aliquot",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="box",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="box",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="boxitem",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="boxitem",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="boxtype",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="boxtype",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="consignee",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="consignee",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalaliquot",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalaliquot",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalaliquot",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalbox",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalbox",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalbox",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalboxitem",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalboxitem",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalboxitem",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalconsignee",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalconsignee",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalconsignee",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalmanifest",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalmanifest",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalmanifest",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="historicalshipper",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalshipper",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="historicalshipper",
            name="history_change_reason",
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name="manifest",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="manifest",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="manifestitem",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="manifestitem",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="shipper",
            name="device_created",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AddField(
            model_name="shipper",
            name="device_modified",
            field=models.CharField(blank=True, max_length=10),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="hostname_created",
            field=models.CharField(
                blank=True,
                default=_socket.gethostname,
                help_text="System field. (modified on create only)",
                max_length=60,
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="user_created",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user created",
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="user_modified",
            field=edc_model_fields.fields.userfield.UserField(
                blank=True,
                help_text="Updated by admin.save_model",
                max_length=50,
                verbose_name="user modified",
            ),
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_appointment", "0030_alter_appointment_appt_status_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalappointment",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical appointment",
                "verbose_name_plural": "historical appointments",
            },
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]



# Generated by Django 2.1 on 2018-09-21 20:45

import uuid

import _socket
import django.db.models.deletion
import django_audit_fields.fields.uuid_auto_field
import django_crypto_fields.fields.encrypted_text_field
import django_revision.revision_field
import simple_history.models
from django.conf import settings
from django.db import migrations, models

import edc_model_fields.fields.hostname_modification_field
import edc_model_fields.fields.userfield
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = [migrations.swappable_dependency(settings.AUTH_USER_MODEL)]

    operations = [
        migrations.CreateModel(
            name="DataRequest",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=25)),
                ("description", models.TextField(null=True)),
                ("decrypt", models.BooleanField(default=False)),
                (
                    "export_format",
                    models.CharField(choices=[("CSV", "CSV")], default="CSV", max_length=25),
                ),
                (
                    "models",
                    models.TextField(
                        help_text='List one table per line, no commas. Use "label lower" format.'
                    ),
                ),
            ],
            options={"ordering": ("name",)},
        ),
        migrations.CreateModel(
            name="DataRequestHistory",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("archive_filename", models.CharField(max_length=200, null=True)),
                ("emailed_to", models.EmailField(max_length=254, null=True)),
                ("emailed_datetime", models.DateTimeField(null=True)),
                ("summary", models.TextField(null=True)),
                (
                    "exported_datetime",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "data_request",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_export.DataRequest",
                    ),
                ),
            ],
            options={"ordering": ("-exported_datetime",)},
        ),
        migrations.CreateModel(
            name="ExportReceipt",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "export_uuid",
                    models.UUIDField(
                        editable=False, help_text="system field for export tracking."
                    ),
                ),
                ("model", models.CharField(max_length=64)),
                ("tx_pk", models.CharField(max_length=36)),
                ("timestamp", models.CharField(max_length=50, null=True)),
                (
                    "received_datetime",
                    models.DateTimeField(help_text="date ACK received", null=True),
                ),
            ],
            options={"ordering": ("-timestamp",)},
        ),
        migrations.CreateModel(
            name="FileHistory",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "history_uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        help_text="system field for export tracking.",
                        unique=True,
                    ),
                ),
                ("model", models.CharField(max_length=50)),
                (
                    "export_uuid_list",
                    models.TextField(
                        help_text="list of export_uuid's of model app_label.model_name",
                        null=True,
                    ),
                ),
                (
                    "pk_list",
                    models.TextField(
                        help_text="list of pk's of model app_label.model_name",
                        null=True,
                    ),
                ),
                (
                    "exit_message",
                    models.CharField(
                        help_text="exit message from the export_transactions command",
                        max_length=250,
                    ),
                ),
                (
                    "exit_status",
                    models.IntegerField(
                        help_text="0=success, 1=failed from the export_transactions command",
                        null=True,
                    ),
                ),
                (
                    "filename",
                    models.CharField(help_text="original filename on export", max_length=250),
                ),
                (
                    "file_contents",
                    models.TextField(
                        help_text="save contents of file as a list of rows", null=True
                    ),
                ),
                (
                    "exported",
                    models.BooleanField(default=False, help_text="exported to a file"),
                ),
                ("exported_datetime", models.DateTimeField(null=True)),
                ("notification_plan_name", models.CharField(max_length=50, null=True)),
                (
                    "sent",
                    models.BooleanField(
                        default=False, help_text="export file sent to recipient"
                    ),
                ),
                ("sent_datetime", models.DateTimeField(null=True)),
                (
                    "received",
                    models.BooleanField(
                        default=False,
                        help_text="have received an ACK from the recipient",
                    ),
                ),
                ("received_datetime", models.DateTimeField(null=True)),
                (
                    "closed",
                    models.BooleanField(default=False, help_text="exported, sent, received"),
                ),
                ("closed_datetime", models.DateTimeField(null=True)),
            ],
            options={"ordering": ("-sent_datetime",)},
        ),
        migrations.CreateModel(
            name="HistoricalDataRequest",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("name", models.CharField(max_length=25)),
                ("description", models.TextField(null=True)),
                ("decrypt", models.BooleanField(default=False)),
                (
                    "export_format",
                    models.CharField(choices=[("CSV", "CSV")], default="CSV", max_length=25),
                ),
                (
                    "models",
                    models.TextField(
                        help_text='List one table per line, no commas. Use "label lower" format.'
                    ),
                ),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                ("history_date", models.DateTimeField()),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        primary_key=True, serialize=False
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical data request",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalExportReceipt",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "export_uuid",
                    models.UUIDField(
                        editable=False, help_text="system field for export tracking."
                    ),
                ),
                ("model", models.CharField(max_length=64)),
                ("tx_pk", models.CharField(max_length=36)),
                ("timestamp", models.CharField(max_length=50, null=True)),
                (
                    "received_datetime",
                    models.DateTimeField(help_text="date ACK received", null=True),
                ),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                ("history_date", models.DateTimeField()),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        primary_key=True, serialize=False
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical export receipt",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalFileHistory",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "history_uuid",
                    models.UUIDField(
                        db_index=True,
                        default=uuid.uuid4,
                        editable=False,
                        help_text="system field for export tracking.",
                    ),
                ),
                ("model", models.CharField(max_length=50)),
                (
                    "export_uuid_list",
                    models.TextField(
                        help_text="list of export_uuid's of model app_label.model_name",
                        null=True,
                    ),
                ),
                (
                    "pk_list",
                    models.TextField(
                        help_text="list of pk's of model app_label.model_name",
                        null=True,
                    ),
                ),
                (
                    "exit_message",
                    models.CharField(
                        help_text="exit message from the export_transactions command",
                        max_length=250,
                    ),
                ),
                (
                    "exit_status",
                    models.IntegerField(
                        help_text="0=success, 1=failed from the export_transactions command",
                        null=True,
                    ),
                ),
                (
                    "filename",
                    models.CharField(help_text="original filename on export", max_length=250),
                ),
                (
                    "file_contents",
                    models.TextField(
                        help_text="save contents of file as a list of rows", null=True
                    ),
                ),
                (
                    "exported",
                    models.BooleanField(default=False, help_text="exported to a file"),
                ),
                ("exported_datetime", models.DateTimeField(null=True)),
                ("notification_plan_name", models.CharField(max_length=50, null=True)),
                (
                    "sent",
                    models.BooleanField(
                        default=False, help_text="export file sent to recipient"
                    ),
                ),
                ("sent_datetime", models.DateTimeField(null=True)),
                (
                    "received",
                    models.BooleanField(
                        default=False,
                        help_text="have received an ACK from the recipient",
                    ),
                ),
                ("received_datetime", models.DateTimeField(null=True)),
                (
                    "closed",
                    models.BooleanField(default=False, help_text="exported, sent, received"),
                ),
                ("closed_datetime", models.DateTimeField(null=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                ("history_date", models.DateTimeField()),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        primary_key=True, serialize=False
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical file history",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalObjectHistory",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                (
                    "exported",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        help_text="system field for export tracking. considered 'exported' if both sent and received.",
                    ),
                ),
                (
                    "export_change_type",
                    models.CharField(
                        choices=[("I", "Insert"), ("U", "Update"), ("D", "Delete")],
                        default="I",
                        editable=False,
                        help_text="system field for export tracking.",
                        max_length=1,
                    ),
                ),
                (
                    "export_uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        help_text="system field for export tracking.",
                    ),
                ),
                ("model", models.CharField(max_length=64)),
                ("tx_pk", models.UUIDField()),
                (
                    "tx",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True, help_text=" (Encryption: AES local)", max_length=71
                    ),
                ),
                ("exported_datetime", models.DateTimeField()),
                (
                    "timestamp",
                    models.CharField(db_index=True, max_length=50, null=True),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("New", "New"),
                            ("exported", "Exported"),
                            ("closed", "Closed"),
                            ("cancelled", "Cancelled"),
                        ],
                        default="New",
                        help_text="exported by export_transactions, closed by import_receipts",
                        max_length=15,
                    ),
                ),
                (
                    "received",
                    models.BooleanField(default=False, help_text="True if ACK received"),
                ),
                (
                    "received_datetime",
                    models.DateTimeField(help_text="date ACK received", null=True),
                ),
                (
                    "is_ignored",
                    models.BooleanField(default=False, help_text="Ignore if update"),
                ),
                ("is_error", models.BooleanField(default=False)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                ("history_date", models.DateTimeField()),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        primary_key=True, serialize=False
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical object history",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalPlan",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("name", models.CharField(db_index=True, max_length=25)),
                ("model", models.CharField(max_length=50)),
                (
                    "field_names",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                (
                    "header_labels",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                ("lookups", models.TextField(blank=True, max_length=500, null=True)),
                (
                    "model_type",
                    models.CharField(
                        choices=[
                            ("consent", "Consent"),
                            ("crf", "CRF"),
                            ("locator", "Locator"),
                            ("plain", "Plain"),
                        ],
                        default="plain",
                        max_length=25,
                    ),
                ),
                (
                    "excluded_field_names",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                (
                    "header",
                    models.BooleanField(default=True, verbose_name="Include header"),
                ),
                (
                    "encrypt",
                    models.BooleanField(default=True, verbose_name="Mask encrypted values"),
                ),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                ("history_date", models.DateTimeField()),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        primary_key=True, serialize=False
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name": "historical plan",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="ObjectHistory",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "exported",
                    models.BooleanField(
                        default=False,
                        editable=False,
                        help_text="system field for export tracking. considered 'exported' if both sent and received.",
                    ),
                ),
                (
                    "export_change_type",
                    models.CharField(
                        choices=[("I", "Insert"), ("U", "Update"), ("D", "Delete")],
                        default="I",
                        editable=False,
                        help_text="system field for export tracking.",
                        max_length=1,
                    ),
                ),
                (
                    "export_uuid",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        help_text="system field for export tracking.",
                    ),
                ),
                ("model", models.CharField(max_length=64)),
                ("tx_pk", models.UUIDField()),
                (
                    "tx",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True, help_text=" (Encryption: AES local)", max_length=71
                    ),
                ),
                ("exported_datetime", models.DateTimeField()),
                (
                    "timestamp",
                    models.CharField(db_index=True, max_length=50, null=True),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("New", "New"),
                            ("exported", "Exported"),
                            ("closed", "Closed"),
                            ("cancelled", "Cancelled"),
                        ],
                        default="New",
                        help_text="exported by export_transactions, closed by import_receipts",
                        max_length=15,
                    ),
                ),
                (
                    "received",
                    models.BooleanField(default=False, help_text="True if ACK received"),
                ),
                (
                    "received_datetime",
                    models.DateTimeField(help_text="date ACK received", null=True),
                ),
                (
                    "is_ignored",
                    models.BooleanField(default=False, help_text="Ignore if update"),
                ),
                ("is_error", models.BooleanField(default=False)),
            ],
            options={"ordering": ("-timestamp",)},
        ),
        migrations.CreateModel(
            name="Plan",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("name", models.CharField(max_length=25, unique=True)),
                ("model", models.CharField(max_length=50)),
                (
                    "field_names",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                (
                    "header_labels",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                ("lookups", models.TextField(blank=True, max_length=500, null=True)),
                (
                    "model_type",
                    models.CharField(
                        choices=[
                            ("consent", "Consent"),
                            ("crf", "CRF"),
                            ("locator", "Locator"),
                            ("plain", "Plain"),
                        ],
                        default="plain",
                        max_length=25,
                    ),
                ),
                (
                    "excluded_field_names",
                    models.TextField(blank=True, max_length=500, null=True),
                ),
                (
                    "header",
                    models.BooleanField(default=True, verbose_name="Include header"),
                ),
                (
                    "encrypt",
                    models.BooleanField(default=True, verbose_name="Mask encrypted values"),
                ),
            ],
            options={
                "ordering": ("-modified", "-created"),
                "get_latest_by": "modified",
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="UploadExportReceiptFile",
            fields=[
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    edc_model_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    edc_model_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                (
                    "export_receipt_file",
                    models.FileField(upload_to="media/edc_export/uploads"),
                ),
                (
                    "file_name",
                    models.CharField(editable=False, max_length=50, null=True, unique=True),
                ),
                ("app_label", models.CharField(max_length=50)),
                ("model_name", models.CharField(max_length=50)),
                ("accepted", models.IntegerField(default=0, editable=False)),
                ("duplicate", models.IntegerField(default=0, editable=False)),
                ("total", models.IntegerField(default=0, editable=False)),
                ("errors", models.TextField(editable=False, null=True)),
                ("receipt_datetime", models.DateTimeField(editable=False, null=True)),
            ],
            options={"ordering": ("-created",)},
        ),
    ]



# Generated by Django 4.2.3 on 2023-08-02 23:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_refusal", "0006_alter_subjectrefusal_managers"),
    ]

    operations = [
        migrations.AddField(
            model_name="refusalreasons",
            name="extra_value",
            field=models.CharField(max_length=250, null=True),
        ),
    ]



from __future__ import annotations

import os
import shutil
import sys
from typing import TYPE_CHECKING

from django.utils import timezone

from edc_export.utils import get_base_dir

if TYPE_CHECKING:
    from datetime import datetime

    from django.contrib.auth.models import User


class FilesArchiver:
    """Archives a folder of CSV files using make_archive."""

    def __init__(
        self,
        path: str = None,
        exported_datetime: datetime = None,
        user: User = None,
        date_format: str = None,
        verbose: bool | None = None,
    ):
        self.exported_datetime: datetime = exported_datetime or timezone.now()
        formatted_date: str = self.exported_datetime.strftime(date_format)
        base_dir: str = get_base_dir()
        archive_name: str = os.path.join(path, f"{user.username}_{formatted_date}")
        self.archive_filename: str = shutil.make_archive(
            archive_name, "zip", root_dir=path, base_dir=base_dir
        )
        if verbose:
            sys.stdout.write(f"\nExported archive to {self.archive_filename}.\n")
